package edu.sc.seis.seisFile.quakeml;

import java.util.ArrayList;
import java.util.List;

import javax.xml.stream.XMLEventReader;
import javax.xml.stream.XMLStreamException;
import javax.xml.stream.events.StartElement;
import javax.xml.stream.events.XMLEvent;

import edu.sc.seis.seisFile.stationxml.NetworkIterator;
import edu.sc.seis.seisFile.stationxml.StationXMLException;
import edu.sc.seis.seisFile.stationxml.StaxUtil;


public class CreationInfo {
    
    public CreationInfo(final XMLEventReader reader) throws XMLStreamException, StationXMLException {
        StaxUtil.skipToStartElement(reader);
        StartElement startE = StaxUtil.expectStartElement(QuakeMLTagNames.QUAKEML, reader);
        while (reader.hasNext()) {
            XMLEvent e = reader.peek();
            if (e.isStartElement()) {
                String elName = e.asStartElement().getName().getLocalPart();
                if (elName.equals(QuakeMLTagNames.agencyID)) {
                    agencyID = StaxUtil.pullText(reader, QuakeMLTagNames.agencyID);
                } else if (elName.equals(QuakeMLTagNames.agencyURI)) {
                    agencyURI = StaxUtil.pullText(reader, QuakeMLTagNames.agencyURI);
                } else if (elName.equals(QuakeMLTagNames.author)) {
                    author = StaxUtil.pullText(reader, QuakeMLTagNames.author);
                } else if (elName.equals(QuakeMLTagNames.authorURI)) {
                    authorURI = StaxUtil.pullText(reader, QuakeMLTagNames.authorURI);
                } else if (elName.equals(QuakeMLTagNames.creationTime)) {
                    creationTime = StaxUtil.pullText(reader, QuakeMLTagNames.creationTime);
                } else if (elName.equals(QuakeMLTagNames.version)) {
                    version = StaxUtil.pullText(reader, QuakeMLTagNames.version);
                } else {
                    StaxUtil.skipToMatchingEnd(reader);
                }
            } else if (e.isEndElement()) {
                return;
            } else {
                e = reader.nextEvent();
            }
        }
    }
    
    
    public String getAgencyID() {
        return agencyID;
    }

    
    public String getAgencyURI() {
        return agencyURI;
    }

    
    public String getAuthor() {
        return author;
    }

    
    public String getAuthorURI() {
        return authorURI;
    }

    
    public String getCreationTime() {
        return creationTime;
    }

    
    public String getVersion() {
        return version;
    }

    String agencyID, agencyURI, author, authorURI, creationTime, version;
}
