package edu.sc.seis.seisFile.quakeml;

import java.util.ArrayList;
import java.util.List;

import javax.xml.stream.XMLEventReader;
import javax.xml.stream.XMLStreamException;
import javax.xml.stream.events.StartElement;
import javax.xml.stream.events.XMLEvent;

import edu.sc.seis.seisFile.stationxml.StationXMLException;
import edu.sc.seis.seisFile.stationxml.StaxUtil;


public class Event {
    
    public Event(final XMLEventReader reader) throws XMLStreamException, StationXMLException {
        this.reader = reader;
        StaxUtil.skipToStartElement(reader);
        StartElement startE = StaxUtil.expectStartElement(QuakeMLTagNames.event, reader);
        while (reader.hasNext()) {
            XMLEvent e = reader.peek();
            if (e.isStartElement()) {
                String elName = e.asStartElement().getName().getLocalPart();
                if (elName.equals(QuakeMLTagNames.description)) {
                    descriptionList.add(new EventDescription(reader));
                } else if (elName.equals(QuakeMLTagNames.comment)) {
                    commentList.add(new Comment(reader));
                    /*
                } else if (elName.equals(QuakeMLTagNames.focalMechanism)) {
                    focalMechanismList.add(new FocalMechanism(reader));
                } else if (elName.equals(QuakeMLTagNames.amplitude)) {
                    amplitudeList.add(new Amplitude(reader));
                } else if (elName.equals(QuakeMLTagNames.magnitude)) {
                    magnitudeList.add(new Magnitude(reader));
                } else if (elName.equals(QuakeMLTagNames.stationMagnitude)) {
                    stationMagnitudeList.add(new StationMagnitude(reader));
                    */
                } else if (elName.equals(QuakeMLTagNames.origin)) {
                    originList.add(new Origin(reader));
                    /*
                } else if (elName.equals(QuakeMLTagNames.pick)) {
                    pickList.add(new Pick(reader));
                    */
                } else if (elName.equals(QuakeMLTagNames.preferredOriginID)) {
                    preferredOriginID = StaxUtil.pullText(reader, QuakeMLTagNames.preferredOriginID);
                } else if (elName.equals(QuakeMLTagNames.preferredMagnitudeID)) {
                    preferredMagnitudeID = StaxUtil.pullText(reader, QuakeMLTagNames.preferredMagnitudeID);
                } else if (elName.equals(QuakeMLTagNames.preferredFocalMechanismID)) {
                    preferredFocalMechanismID = StaxUtil.pullText(reader, QuakeMLTagNames.preferredFocalMechanismID);
                } else {
                    StaxUtil.skipToMatchingEnd(reader);
                }
            } else if (e.isEndElement()) {
                return;
            } else {
                e = reader.nextEvent();
            }
        }
    }

    String preferredOriginID, preferredMagnitudeID, preferredFocalMechanismID;
    
    List<EventDescription> descriptionList = new ArrayList<EventDescription>();

    List<Comment> commentList = new ArrayList<Comment>();
    List<FocalMechanism> focalMechanismList = new ArrayList<FocalMechanism>();
    List<Amplitude> amplitudeList = new ArrayList<Amplitude>();
    /*
    List<Magnitude> magnitudeList = new ArrayList<Magnitude>();
    List<StationMagnitude> stationMagnitudeList = new ArrayList<StationMagnitude>();
    List<Pick> pickList = new ArrayList<Pick>();
    */
    List<Origin> originList = new ArrayList<Origin>();
    
    XMLEventReader reader;
}
